#!/usr/local/bin/perl
#---*----*----*----*----*----*----*----*----*----*----*----*----*----*----*----*----*----*
#
# GUI - Example BMEcat-XML-Generator
#
# BMEcat is a trademark of BME - Bundesverband Materialwirtschaft, Einkauf und Logistik e.V.
#
#---*----*----*----*----*----*----*----*----*----*----*----*----*----*----*----*----*----*

use strict vars;
use IO::File;
use File::Basename;
use Tk;
use Tk::DirTree;
use Tk::NoteBook;
use Tk::BrowseEntry;
use Tk::ErrorDialog;

use XML::BMEcat;

Main();

#----*----*----*----*----*----*----*----*----*----*----*-  M A I N  --*----*----*----*----*

sub Main {

   my $BMEcat = XML::BMEcat->new();

   my $Header = $BMEcat->creatHeader();

   my ( $APPL_CFG, $r_info )  = Init();

   my $TOP = MainWindow->new;
   $APPL_CFG->{'TOP'} = $TOP;
   $TOP->title('MiniKatMenu');
   $TOP->iconname(basename($0));

   my $menubar = $TOP->Frame(qw/-relief raised -background DarkGray -bd 2/);
   $menubar->pack(-side => 'top', -fill => 'x');

   my $workarea = $TOP->Frame(-width => 100, -height => 350);

   my $label = $workarea->Label(-text => "Welcome to " . basename($0),
                                -font => 'Arial-12')->pack(-pady => 150);

   $workarea->pack(-side => 'top', -fill => 'both', -expand => 1);

   my @menu_button_list = ();
   push @menu_button_list,
   mkmb($APPL_CFG, $menubar, 'File', 0, 'File related stuff',
        [
           ['Change WorkDir', sub {  SelectDir($APPL_CFG) },           0],

           ['Save XML',       sub {  ExportXML($APPL_CFG, $BMEcat)  }, 0],

           ['Exit',           sub {  exit },                           0],
        ]);

   push @menu_button_list,
   mkmb($APPL_CFG, $menubar, 'Edit', 0, 'Edit Your changes',
        [
           ['MiniKatalog',    sub {  		MiniKatalog(
						$APPL_CFG,
						$BMEcat,
						$Header);
				  },			               0],

           ['Header',         sub { destroy $label, EditHeader( $APPL_CFG,
						$workarea,
						$Header,
						$r_info) },             0],
        ]);

   push @menu_button_list,
   mkmb($APPL_CFG, $menubar, 'Extra', 0, 'Various cool functions',
        [
           ['Validate *',     sub { NotDone($APPL_CFG) },               0]
        ]);

   push @menu_button_list,
   mkmb($APPL_CFG, $menubar, 'Help', 0, 'There when you need it',
        [
           ['About..',        sub { About($APPL_CFG) },                 0],
        ]);

   $menu_button_list[$#menu_button_list]->pack(-side => 'right');

   my $feedback = $TOP->Frame();
   $feedback->pack(-side => 'bottom', -fill => 'x');
   $APPL_CFG->{'STATUS'} = $feedback->Text(
       -relief       => 'sunken',
       -height       => 1,
       -background   => 'Darkgray',
       -borderwidth  => 2,
       );

   $APPL_CFG->{'STATUS'}->pack(-side => 'bottom', -fill => 'x', -expand => 1);

   MainLoop;
}

#---*----*----*----*----*----*----*----*----*----*----*-  S U B S  --*----*----*

sub Init {

   my ($sec,$min,$hour,$mday,$mon,$year) = localtime(time);

   my $r_info = {
         'General'  => ['GENERATOR_INFO'	=> "Created with " . basename($0),
         		'LANGUAGE'		=> "DEU",
         		'CATALOG_ID'		=> "6",
         		'CATALOG_VERSION'	=> "100",
			'PREV_VERSION'		=> "",
         		'CATALOG_NAME'		=> "",
         		'DATE'			=> sprintf ("%4d-%02d-%02d",1900+$year,++$mon,$mday),
         		'TIME'			=> sprintf ("%02d:%02d:%02d",$hour,$min,$sec),
         		'CURRENCY'		=> "DEM",
         		'MIME_ROOT'		=> "//Bilder"],
         'Buyer'    => ['BUYER_ID'		=> "0815",
         		'BUYER_NAME'		=> FOO,
         		'NAME'			=> "FOO CORPORATION",
         		'NAME2'			=> "",
         		'CONTACT'		=> "",
         		'STREET'		=> "Business Street 17",
         		'ZIP'			=> 01234,
         		'CITY'			=> 'New York',
         		'COUNTRY'		=> USA,
         		'PHONE'			=> "",
         		'FAX'			=> "",
         		'EMAIL'			=> "info\@foo-bussines.com",
         		'URL'			=> "http://www.foo-bussines.com" ],
         'Agreement'=> ['AGREEMENT_ID'		=> "4711",
         		'AGREEMENT_start_date'	=> "2000-01-01",
         		'AGREEMENT_end_date'	=> "2000-12-31"],
         'Supplier' => ['SUPPLIER_ID'		=> "1",
			'SUPPLIER_ID_type'	=> 'buyer_specific',
         		'SUPPLIER_NAME'		=> "fischerwerke",
         		'NAME'			=> "Artur Fischer Gmbh & Co KG",
         		'STREET'		=> "Weinhalde 14 - 18",
         		'ZIP'			=> "72178",
         		'CITY'			=> "Waldachtal",
         		'COUNTRY'		=> "Germany",
         		'PHONE'			=> "+49 7443 12 0",
         		'FAX'			=> "+49 7443 12 42 22",
         		'EMAIL'			=> "info\@fischerwerke.com",
         		'URL'			=> "http://www.fischerwerke.com"],
         'Config'   => ['FEATURE_SYSTEM_NAME'	=> 'Generic',
                        'GROUP_SYSTEM_ID'	=> "01-1-00/01",
                        'CHAR_SET'              => "ISO-8859-1",
                        'DTD'                   => "bmecat_new_catalog.dtd"
                       ]
   };

   my $APPL_CFG = {     'CATALOG_GROUP_SYSTEM'        => 0,
                        'ARTIKEL_TO_CATALOGGROUP_MAP' => 0
                  };


   return $APPL_CFG, $r_info;

}  # end Init


sub EditHeader {
   my ($APPL_CFG, $workarea, $Header, $r_info) = @_;
 
   my $Notebook = $workarea->NoteBook(-ipadx => 6, -ipady => 6,
                             -background  => 'DarkGray'
                  );

   my %Notebook = {};
   foreach my $page (qw/General Buyer Supplier Agreement Config/) {

      $Notebook{$page} = EditNotebookPage($Header, $r_info, $Notebook, $page);
   }


   $Notebook{'Agreement'}->Checkbutton(-text       => "Add agreement-info to head ?",
                                       -variable   => \$Header->{'AGREEMENT'}
                                      )->pack(side => 'bottom');

   my (@pl) = qw/-side top -fill both -expand 1/;
   foreach my $cb (qw/ FEATURE_SYSTEM CATALOG_GROUP_SYSTEM ARTIKEL_TO_CATALOGGROUP_MAP/) {

      my $cb_area = $Notebook{'General'}->Frame()->pack(@pl);

      $cb_area->Label(-text              => "Add section: ",
                      -width             => 25,
    	         	-anchor          => 'center'
                     )->pack(-side       => 'left', -anchor => 'w');

      $cb_area->Checkbutton(-text        => "$cb",
                       -indicatoron => 0,
                       -selectcolor => 'DarkGray',
                       -variable    => \$APPL_CFG->{"$cb"}
                      )->pack(-side => 'right', -fill => 'both', -expand => 'x');
   }

   $Header->setTransaction('T_NEW_CATALOG');

   $APPL_CFG->{'TOP'}->title("MiniKatMenu: $Header->{'General'}->{'CATALOG_NAME'}");

   $APPL_CFG->{'EDIT_HEADER'} = 1;
}


sub EditNotebookPage {
   my ($INFO, $r_info, $Notebook, $Section) = @_;

   my ($key, $val);

   my $notebook_p = $Notebook->add("$Section",
                 -label => "$Section",
                 -underline => 0,
                 );

   $Notebook->pack(-expand => "yes",
		 -fill => "both",
		 -padx => 5, -pady => 5,
		 -side => "top");

   my $ScrText = $notebook_p->Text(
        -width        => 40,
	-background   => 'Gray',
        -wrap         => 'none'
      )->pack(-expand => 1, -fill => 'both');

   while (($key, $val) = splice @{$r_info->{$Section}}, 0, 2) {

        my $w = $ScrText->Label(-text       => "$key:",
                                -relief     => 'groove',
                                -background => 'Gray',
                                -width      => 25);

        $ScrText->windowCreate('end', -window => $w);

        $INFO->{$Section}->{$key} = $val;

        $w = $ScrText->Entry(-width        => 50,
                             -textvariable => \$INFO->{$Section}->{$key},
             );

        $ScrText->windowCreate('end', -window => $w);

        $ScrText->insert('end', "\n");
   }

   $ScrText->configure(-state => 'disabled');

   return $notebook_p;
}


sub MiniKatalog {
   my ($APPL_CFG, $BMEcat, $Header) = @_;


   $Header->setTransaction('T_NEW_CATALOG', [ 'prev_version' => '1.0' ]);

############################### Merkmalssystem #################################

   my $FeatureSystem = $BMEcat->creatFeatureSystem();

   $FeatureSystem->addFeatureGroup(
	'116',
	'Lnge'						=> "mm",
	'min. Dicke bis zu ersten Trgerschichten'	=> "mm",
	'Typ'						=> ""
   );


############################### Katalogstruktur ################################

   $Header->setConfigInfo('GROUP_SYSTEM_ID'	=> '01-1-00/01');

   my $GroupSystem = $BMEcat->creatGroupSystem();

   my $CatalogGroup = $GroupSystem->creatCatalogGroup('02');
   $CatalogGroup->setData(
	'PARENT'	=>	0,
	'NAME'		=>	"fischer Befestigungskatalog",
	'SORT'		=>	5
   );

   $CatalogGroup = $GroupSystem->creatCatalogGroup('04');
   $CatalogGroup->setData(
	'PARENT'	=>	2,
	'NAME'		=>	"Allgemeine Befestigungen",
	'SORT'		=>	5
   );

   $CatalogGroup = $GroupSystem->creatCatalogGroup('06');
   $CatalogGroup->setData(
	'PARENT'	=>	2,
	'NAME'		=>	"Hohlraumbefestigungen",
	'SORT'		=>	10
   );

   $CatalogGroup = $GroupSystem->creatCatalogGroup('08');
   $CatalogGroup->setData(
	'PARENT'	=>	4,
	'NAME'		=>	"fischer Gipskartondbel GK",
	'SORT'		=>	5,
	'LEAF'		=>      1
   );
   $CatalogGroup->addDescription("inkl. 1 Setzwerkzeug");
   $CatalogGroup->addMime('image/jpg', "fis101274.jpg", "normal");

   $CatalogGroup = $GroupSystem->creatCatalogGroup('10');
   $CatalogGroup->setData(
	'PARENT'	=>	4,
	'NAME'		=>	"fischer Gipskartondbel GKM",
	'SORT'		=>	10,
	'LEAF'		=>      1
   );


################################### Artikel ####################################

   my $ArticleSystem = $BMEcat->creatArticleSystem();

   my $Article = $ArticleSystem->creatArticle('52389');

   $Article->setMainInfo(
	'mode'		=>	'new',
	'SUPPLIER_AID'  =>	'52389'
   );

   $Article->setFeatureGroup('116');

   $Article->setFeatureValues(
		'22', 
		'25',
		'GK'
	);

   $Article->addMime('image/jpg', '4006209523896.jpg', 'normal');


   my $DESCRIPTION_LONG = <<'--end--';
Der fischer Gipskartondbel GK ist ein Spezialdbel, der
mit dem beigefgten Setzwerkzeug nicht hinter der Platte,
sondern formschlssig in die Gipskartonplatte eingedreht
wird. Dadurch wird hinter der Platte nur wenig Platz bentigt.
--end--

   $Article->setDetails(
	'DESCRIPTION_SHORT'	=> 'Der Schnellmontagedbel fr Gipskarton',
	'DESCRIPTION_LONG'	=> $DESCRIPTION_LONG,
	'EAN'			=> '4006209523896'
   );

   $Article->setOrderDetails(
	'ORDER_UNIT'		=> "Pkg.",
	'CONTENT_UNIT'		=> "Stk.",
	'NO_CU_PER_OU'		=> 100
   );

   $Article->setPriceDetails(
	'valid_start_date'	=> '1999-10-01',
	'valid_end_date'	=> '2000-09-31'
   );


   $Article->addPrice(
	'price_type'		=> 'net_list',
	'PRICE_AMOUNT'		=> '50,00',
	'PRICE_CURRENCY'	=> 'EUR'
   );


########################## Artikel-Gruppenzuweisung ############################

   $Article->map2Group("08");


################################## Schlu ######################################

   Msg($APPL_CFG, "MiniKatalog ready!");
}


sub ExportXML {
   my ($APPL_CFG, $BMEcat) = @_;

   $BMEcat->setOutfile(fileDialog($APPL_CFG, "save"));

   $BMEcat->writeHeader();                             # BMEcat-Kopf schreiben

   $BMEcat->writeFeatureSystem()
       if $APPL_CFG->{'FEATURE_SYSTEM'};               # Merkmalsleistensystem schreiben

   $BMEcat->writeGroupSystem()
       if $APPL_CFG->{'CATALOG_GROUP_SYSTEM'};         # Katalogstruktur schreiben

   $BMEcat->writeArticleSystem();                      # Artikeleintrge

   $BMEcat->writeArticleGroupMap()
       if $APPL_CFG->{'ARTIKEL_TO_CATALOGGROUP_MAP'};  # Artikel-Grundlistenzuweisung

   $BMEcat->writeTail;

   Msg($APPL_CFG, "ExportXML ready!");

   exit;
}


sub SelectDir {
   my $APPL_CFG = shift;

   my $top = MainWindow->new;

   my $dl  = $top->Scrolled('DirTree')->pack(
                                            -expand => 1,
                                            -fill   => 'both',
                                            -ipadx  => '1i',
                                            -ipady  => '1i'
                                        );
   $dl->configure(-command => sub {
          $APPL_CFG->{'HOME'} = $_[0];
          chdir $APPL_CFG->{'HOME'};
          Msg($APPL_CFG, "WorkDir changed to: $APPL_CFG->{'HOME'} !");
   });
}


sub About {
   my $APPL_CFG = shift;
   my $DIALOG_ABOUT;

   if (not Exists($DIALOG_ABOUT)) {
      $DIALOG_ABOUT = $APPL_CFG->{'TOP'}->Dialog(
          -title          => 'About MiniKatMenu',
          -bitmap         => 'info',
          -default_button => 'OK',
          -buttons        => ['OK'],
          -text           => '',
      );
      $DIALOG_ABOUT->configure(
            -wraplength => '4i',
            -text       => "         This is a prototypical Perl/Tk-application.\n\n" .
                           "Written to demonstrate:\n" .
                           "   - The possible functions of a BMEcat-XML-Generator,\n" .
                           "   - The power of Perl.\n\n" .
                           "By Frank-P. Reich (fp\$, fpreich\@cpan.org), I/2000\n",
        );
   }
   $DIALOG_ABOUT->Show;
}


sub mkmb {
    # (Ripped from nTk examples)
    # Make a Menubutton widget; note that the menu is automatically created.  
    # We maintain a list of the Menubutton references since some callers 
    # need to refer to the Menubutton, as well as to suppress stray name 
    # warnings with Perl -w.
    # Thanks to Gurusamy Sarathy (gsar@engin.umich.edu)
    #
    # Tidied up by SOL and FP$.

    my($APPL_CFG, $mb0, $mb_label, $mb_label_underline, $mb_msg, $mb_list_ref) = @_;
    my $mb = $mb0->Menubutton(
        -text       => $mb_label, 
	-underline  => $mb_label_underline,
	-background => 'DarkGray',
        -foreground => 'Black',
    );
    my($menu) = $mb->Menu(-tearoff => 0);
    $mb->configure(-menu => $menu);

    foreach my $mb_list (@{$mb_list_ref}) {
        $mb->command(
            -label      => $mb_list->[0], 
            -command    => $mb_list->[1] , 
            -underline  => $mb_list->[2],
            -background => 'DarkGray',
            -foreground => 'Black',
        );
    }
    $mb->pack(-side => 'left');
    $APPL_CFG->{'TOP'}->bind($mb, '<Enter>' => sub { Msg($APPL_CFG, $mb_msg) } );
    $APPL_CFG->{'TOP'}->bind($mb, '<Leave>' => sub { ClearMsg($APPL_CFG) } );
    
    return $mb;

} # end mkmb


sub fileDialog {
    my $APPL_CFG  = shift;
    my $operation = shift;
    my $file;
    #   Type names		Extension(s)	Mac File Type(s)
    #
    #---------------------------------------------------------
    my @types =
      (["Text files",           '',             'TEXT'],
       ["Perl Scripts",         '.pl',		'TEXT'],
       ["C Source Files",	['.c', '.h']],
       ["All Source Files",     [qw/.tcl .c .h/]],
       ["Image Files",		'.gif'],
       ["Image Files",		['.jpeg', '.jpg']],
       ["Image Files",   	'',		[qw/GIFF JPEG/]],
       ["All files",		'*']
      );
    if ($operation eq 'open') {
        unshift @types, ["Text files",           [qw/.txt .doc/]];
	$file = $APPL_CFG->{'TOP'}->getOpenFile(-filetypes => \@types);
    } else {
        unshift @types, ["XML files",            '.xml',		'TEXT'];
	$file = $APPL_CFG->{'TOP'}->getSaveFile(-filetypes => \@types,
				-initialfile => 'MiniKatalog',
				-defaultextension => '.xml');
    }
    return $file;
}


sub NotDone {
    my $APPL_CFG = shift;

    Msg ($APPL_CFG, "Not yet implemented.\n");

} # end NotDone


sub ShowMsg {
    my $APPL_CFG = shift;
    my($msg)     = shift;

    $APPL_CFG->{'STATUS'}->insert('1.0', $msg);  

} # end ShowMsg


sub ClearMsg {
    my $APPL_CFG = shift;

    $APPL_CFG->{'STATUS'}->delete('1.0', 'end') if defined $APPL_CFG->{'STATUS'};

} # end ClearMsg


sub Msg {
    my $APPL_CFG = shift;
    my($msg) = shift;   

    ClearMsg($APPL_CFG);
    ShowMsg($APPL_CFG, $msg);

} # end Msg
